# Copyright Visual Collaboration Technologies Inc. All Rights Reserved.
# See LICENSE for details.
"""vcti.deck — generic ordered collection with named items and policy-based compatibility."""

from importlib.metadata import version

from .deck import Deck
from .policy_bound_deck import CompatibilityPolicy, PolicyBoundDeck

__version__ = version("vcti-deck")

__all__ = [
    "__version__",
    "CompatibilityPolicy",
    "Deck",
    "PolicyBoundDeck",
]
